import mugrade
import numpy as np


def test_load_twitter_data_sqlite3():
    cursor = load_twitter_data_sqlite3(sqlite3.connect(":memory:")).cursor()
    with mugrade.test:
        assert cursor.execute(
            "SELECT name FROM sqlite_master WHERE type = 'table';").fetchall() == [('users',), ('edges',), ('tweets',)]
    with mugrade.test:
        assert cursor.execute(
            'SELECT COUNT(*) FROM users;').fetchone() == (12403,)
    with mugrade.test:
        assert cursor.execute(
            "SELECT screen_name FROM users WHERE name='Barack Obama';").fetchone() == ('BarackObama',)
    with mugrade.test:
        assert cursor.execute(
            "SELECT * FROM pragma_table_info('edges') WHERE name='friend';").fetchone() == (1, 'friend', 'TEXT', 0, None, 0)


def submit_load_twitter_data_sqlite3():
    cursor = load_twitter_data_sqlite3(sqlite3.connect(":memory:")).cursor()
    mugrade.submit(cursor.execute(
        "SELECT COUNT(*) FROM pragma_table_info('tweets') WHERE name='screen_name';").fetchone())
    mugrade.submit(cursor.execute(
        'SELECT COUNT(*) FROM tweets;').fetchone())
    mugrade.submit(cursor.execute(
        "SELECT screen_name FROM tweets WHERE retweet_count=131102 LIMIT 1;").fetchone())
    mugrade.submit(cursor.execute(
        "SELECT COUNT(*) FROM edges where friend = 'BillGates';").fetchone())
    mugrade.submit(cursor.execute(
        "SELECT * FROM pragma_table_info('tweets') WHERE name='retweet_count';").fetchone())


def test_trending_tweets():
    cursor = load_twitter_data_sqlite3(sqlite3.connect(':memory:')).cursor()
    with mugrade.test:
        assert list(trending_tweets(cursor, ['Olympics'], 2)) == [("RT @NBCOlympics: The World's Greatest Athlete!\r\n\r\n@AshtonJEaton wins back-to-back #gold medals in the decathlon! #Rio2016 https://t.co/XSDC3j…", 1355),
                                                                  ('RT @usavolleyball: The #USAVwnt defeats #JPN 3-0 to advance to the Semifinals! #TeamUSA #ALLIN #Olympics https://t.co/5qTiM7Sb95', 714)]
    with mugrade.test:
        assert len(list(trending_tweets(cursor, ['Barack', 'Obama'], 6))) == 6
    with mugrade.test:
        assert list(trending_tweets(cursor, ['cancer'], 2)) == [('Hundreds of students sing to cancer-stricken teacher https://t.co/IcLfxENEJf https://t.co/pRF9FBIZjp', 265),
                                                                ('Cancer kills 1 person every minute. Lets find a cure!!! @Whipping_Cancer @jmenatica #KK', 203)]


def submit_trending_tweets():
    cursor = load_twitter_data_sqlite3(sqlite3.connect(':memory:')).cursor()
    mugrade.submit(list(trending_tweets(cursor, ['ESPN'], 2)))
    mugrade.submit(
        len(list(trending_tweets(cursor, ['Artificial', 'Intelligence'], 4))))
    mugrade.submit(list(trending_tweets(cursor, ['Volleyball'], 2)))


def test_num_tweets_in_feed():
    cursor = load_twitter_data_sqlite3(sqlite3.connect(':memory:')).cursor()
    with mugrade.test:
        assert len(num_tweets_in_feed(cursor).fetchall()) == 12403
    with mugrade.test:
        assert list(filter(lambda x: x[0] == "CNNPolitics", num_tweets_in_feed(cursor).fetchall())) == [('CNNPolitics', 80)]


def submit_num_tweets_in_feed():
    cursor = load_twitter_data_sqlite3(sqlite3.connect(':memory:')).cursor()
    mugrade.submit(list(
        filter(lambda x: x[0] == "worthy1961", num_tweets_in_feed(cursor).fetchall())))
    mugrade.submit(
        len(list(filter(lambda x: x[1] > 100, num_tweets_in_feed(cursor).fetchall()))))
    mugrade.submit(sorted(list(num_tweets_in_feed(cursor)),
                          key=lambda x: x[1], reverse=True)[:5])


def test_load_twitter_data_pandas():
    data = load_twitter_data_pandas()
    with mugrade.test:
        assert data[0]["location"].values[:10].tolist() == ['New York, NY', 'New York, NY', '', '', 'DC', '', 'USA', 'US', '', 'New York City']
    with mugrade.test:
        assert data[2]["retweet_count"].dtype == np.int64
    with mugrade.test:
        assert len(data[1]) == 16179


def submit_load_twitter_data_pandas():
    data = load_twitter_data_pandas()
    mugrade.submit(data[2]["created_at"].values[:10].tolist())
    mugrade.submit(data[1]["friend"].dtype)
    mugrade.submit(len(data[2]))


def test_plot_friends_vs_followers():
    plot = plot_friends_vs_followers(load_twitter_data_pandas()[0])
    with mugrade.test:
        assert plot.get_offsets()[:10, :].tolist() == [
            [11397769.0, 42.0], 
            [9954.0, 35.0], 
            [8797.0, 200.0], 
            [63138.0, 79.0], 
            [851876.0, 289.0], 
            [207552.0, 1208.0], 
            [65833.0, 18.0], 
            [999809.0, 2.0], 
            [50517.0, 155.0], 
            [60552.0, 342.0]]
    with mugrade.test:
        assert len(plot.get_offsets()) == 12403


def submit_plot_friends_vs_followers():
    plot = plot_friends_vs_followers(load_twitter_data_pandas()[0])
    mugrade.submit(plot.get_offsets().max(axis=0).data)
    mugrade.submit(plot.get_offsets().min(axis=0).data)
    mugrade.submit(str(type(plot)))


def test_correlation_coefficient():
    user_data = load_twitter_data_pandas()[0]
    with mugrade.test:
        assert np.allclose(np.array([correlation_coefficient(user_data)]), [0.09308641], atol=1e-8)
    with mugrade.test:
        assert np.allclose(np.array([correlation_coefficient(user_data[:50])]), [0.00834681], atol=1e-8)


def submit_correlation_coefficient():
    user_data = load_twitter_data_pandas()[0]
    mugrade.submit(np.array([correlation_coefficient(user_data[:1000])]))
    mugrade.submit(np.array([correlation_coefficient(user_data[1000:])]))


def test_degree_distribution():
    edge_data = load_twitter_data_pandas()[1]
    with mugrade.test:
        assert np.allclose(degree_distribution(edge_data)[0], [2.0, 6.0, 2.0, 5.0, 4.0, 4.0, 3.0, 1.0, 4.0, 148.0], atol=1e-8)
    with mugrade.test:
        assert str(type(degree_distribution(edge_data)[
                   2])) == "<class 'matplotlib.cbook.silent_list'>"


def submit_degree_distribution():
    edge_data = load_twitter_data_pandas()[1]
    mugrade.submit(degree_distribution(edge_data)[1])
